/***************************************************************************
 *   Copyright (C) 2008 by S. MANKOWSKI / G. DE BURE skrooge@mankowski.fr  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>  *
 ***************************************************************************/
/** @file
 * This file defines classes SKGUnitObject.
 *
 * @author Stephane MANKOWSKI / Guillaume DE BURE
 */
#include "skgunitobject.h"
#include "skgunitvalueobject.h"
#include "skgdocumentbank.h"
#include "skgtraces.h"

#include <klocale.h>
#include <kglobal.h>
//#include <kcurrencycode.h>

#include <QRegExp>
#include <math.h>

SKGUnitObject::SKGUnitObject ( const SKGDocument* iDocument, int iID )
        : SKGNamedObject ( iDocument, "v_unit", iID )
{
}

SKGUnitObject::~SKGUnitObject()
{
}

SKGUnitObject::SKGUnitObject ( const SKGUnitObject& iObject )
        : SKGNamedObject ( iObject )
{
}

SKGUnitObject::SKGUnitObject ( const SKGNamedObject& iObject )
{
    if ( iObject.getRealTable() =="unit" )
    {
        copyFrom ( iObject );
    }
    else
    {
        *this=SKGNamedObject::SKGNamedObject ( iObject.getDocument(), "v_unit", iObject.getID() );
    }
}

SKGUnitObject::SKGUnitObject ( const SKGObjectBase& iObject )
{
    if ( iObject.getRealTable() =="unit" )
    {
        copyFrom ( iObject );
    }
    else
    {
        *this=SKGNamedObject::SKGNamedObject ( iObject.getDocument(), "v_unit", iObject.getID() );
    }
}

const SKGUnitObject& SKGUnitObject::operator= ( const SKGObjectBase& iObject )
{
    copyFrom ( iObject );
    return *this;
}

QString SKGUnitObject::getWhereclauseId() const
{
    QString output = SKGObjectBase::getWhereclauseId();
    if ( output.isEmpty() )
    {
        QString name=getName();
        if ( !name.isEmpty() ) output="t_name='"+SKGServices::stringToSqlString ( name ) +'\'';

        QString symbol=getSymbol();
        if ( !symbol.isEmpty() )
        {
            if ( !output.isEmpty() ) output+=" OR ";
            output+="t_symbol='"+SKGServices::stringToSqlString ( symbol ) +'\'';
        }
        if ( !output.isEmpty() ) output='('+output+')';
    }

    return output;
}

QStringList SKGUnitObject::getListofKnownCurrencies()
{
    QStringList output;
    /*    QStringList units=KCurrencyCode::allCurrencyCodesList();
        int nb=units.count();
        for(int i=0; i<nb; ++i)
        {
    	output << KCurrencyCode::currencyCodeToName(units.at(i))+" ("+units.at(i)+")";
        }
        output.sort();*/

    output
    << i18nc ( "Noun, a currency", "Euro (EUR)" )
    << i18nc ( "Noun, a currency", "United States Dollar (USD)" )
    << i18nc ( "Noun, a currency", "French Franc (FRF)" )
    << i18nc ( "Noun, a currency", "Belgium Franc (BEF)" )
    << i18nc ( "Noun, a currency", "Deutsche Mark (DEM)" )
    << i18nc ( "Noun, a currency", "Espana Peseta (ESP)" )
    << i18nc ( "Noun, a currency", "Italian Lira (ITL)" )
    << i18nc ( "Noun, a currency", "Pound Sterling (GBP)" )
    << i18nc ( "Noun, a currency", "Swedish krona (SEK)" )
    << i18nc ( "Noun, a currency", "Canadian Dollar (CAD)" )

    << i18nc ( "Noun, a currency", "CAC 40" )
    << i18nc ( "Noun, a currency", "Dow Jones (DJIA)" )
    << i18nc ( "Noun, a currency", "NASDAQ" )
    << i18nc ( "Noun, a currency", "SBF 120" );
    return output;
}

QString SKGUnitObject::getInternationalCode ( const QString& iUnitName )
{
    QString output=iUnitName;
    QRegExp rx ( ".*\\((.+)\\).*" );
    if ( rx.indexIn ( iUnitName ) !=-1 )
    {
        output=rx.cap ( 1 );
    }

    return output;
}


SKGServices::SKGUnitInfo SKGUnitObject::getUnitInfo ( const QString& iUnitName )
{
    SKGServices::SKGUnitInfo info;

    info.NbDecimal=2;
    info.Value=0;

    //Principal units
    if ( iUnitName==i18nc ( "Noun, a currency", "Euro (EUR)" ) || iUnitName=="EUR" || iUnitName==QString::fromUtf8 ( "€" ) )
    {
        //Euro
        info.Name=i18nc ( "Noun, a currency", "Euro (EUR)" );
        info.Symbol=QString::fromUtf8 ( "€" );
        info.Country=i18nc ( "Noun, a country", "Europe" );
        info.Date=QDate ( 2002,1,1 );
        info.Value=1;
    }
    else if ( iUnitName==i18nc ( "Noun, a currency", "United States Dollar (USD)" ) || iUnitName=="USD" || iUnitName==QString::fromUtf8 ( "$" ) )
    {
        //US Dollar
        info.Name=i18nc ( "Noun, a currency", "United States dollar (USD)" );
        info.Symbol=QString::fromUtf8 ( "$" );
        info.Country=i18nc ( "Noun, a country", "United States" );
        info.Date=QDate ( 1959,1,1 );
        info.Value=1;
    }
    else if ( iUnitName==i18nc ( "Noun, a currency", "Canadian Dollar (CAD)" ) || iUnitName=="CAD" || iUnitName==QString::fromUtf8 ( "C$" ) )
    {
        //US Dollar
        info.Name=i18nc ( "Noun, a currency", "Canadian dollar (CAD)" );
        info.Symbol=QString::fromUtf8 ( "C$" );
        info.Country=i18nc ( "Noun, a country", "Canada" );
        info.Date=QDate ( 1858,1,1 );
        info.Value=1;
    }
    else if ( iUnitName==i18nc ( "Noun, a currency", "Pound Sterling (GBP)" ) || iUnitName=="GBP" || iUnitName==QString::fromUtf8 ( "£" ) )
    {
        //Lyre
        info.Name=i18nc ( "Noun, a currency", "Pound sterling (GBP)" );
        info.Symbol=QString::fromUtf8 ( "£" );
        info.Country=i18nc ( "Noun, a country", "United Kingdom" );
        info.Date=QDate ( 1990,9,8 );
        info.Value=1;
    }
    //Europeen units
    else if ( iUnitName==i18nc ( "Noun, a currency", "French Franc (FRF)" ) || iUnitName=="FRF" || iUnitName=="F" )
    {
        //Franc
        info.Name=i18nc ( "Noun, a currency", "French Franc (FRF)" );
        info.Symbol='F';
        info.Country=i18nc ( "Noun, a country", "France" );
        info.Date=QDate ( 1963,1,1 );
        info.Value=1.0/6.55957;
        info.Parent="EUR";
    }
    else if ( iUnitName==i18nc ( "Noun, a currency", "Deutsche Mark (DEM)" ) || iUnitName=="DEM" || iUnitName=="DM" )
    {
        //Deutch mark
        info.Name=i18nc ( "Noun, a currency", "Deutsche mark (DEM)" );
        info.Symbol="DM";
        info.Country=i18nc ( "Noun, a country", "Germany" );
        info.Date=QDate ( 1948,1,1 );
        info.Value=1.0/1.955083;
        info.Parent="EUR";
    }
    else if ( iUnitName==i18nc ( "Noun, a currency", "Espana Peseta (ESP)" ) || iUnitName=="ESP" || iUnitName=="P" )
    {
        //Pesetas
        info.Name=i18nc ( "Noun, a currency", "Espana Peseta (ESP)" );
        info.Symbol='P';
        info.Country=i18nc ( "Noun, a country", "Spain" );
        info.Date=QDate ( 1868,1,1 );
        info.Value=1.0/166.386;
        info.Parent="EUR";
    }
    else if ( iUnitName==i18nc ( "Noun, a currency", "Italian Lira (ITL)" ) || iUnitName=="ITL" || iUnitName==QString::fromUtf8 ( "£" ) )
    {
        //Lyre
        info.Name=i18nc ( "Noun, a currency", "Italian Lira (ITL)" );
        info.Symbol=QString::fromUtf8 ( "£" );
        info.Country=i18nc ( "Noun, a country", "Italy" );
        info.Date=QDate ( 1970,1,1 );
        info.Value=1.0/1936.27;
        info.Parent="EUR";
    }
    else if ( iUnitName==i18nc ( "Noun, a currency", "Belgium Franc (BEF)" ) || iUnitName=="BEF" || iUnitName=="FB" )
    {
        //Lyre
        info.Name=i18nc ( "Noun, a currency", "Belgium Franc (BEF)" );
        info.Symbol="FB";
        info.Country=i18nc ( "Noun, a country", "Belgium" );
        info.Date=QDate ( 1970,1,1 );
        info.Value=1.0/40.3399;
        info.Parent="EUR";
    }
    else if ( iUnitName==i18nc ( "Noun, a currency", "Swedish krona (SEK)" ) || iUnitName=="SEK" || iUnitName=="KR" )
    {
        //Swedish krona
        info.Name=i18nc ( "Noun, a currency", "Swedish krona (SEK)" );
        info.Symbol="KR";
        info.Country=i18nc ( "Noun, a country", "Sweden" );
        info.Date=QDate ( 1970,1,1 );
        info.Value=-1;
        info.Internet="EURSEK=X";
        info.Parent="EUR";
    }

    //Indexes
    else if ( iUnitName==i18nc ( "Noun, a currency", "CAC 40" ) )
    {
        //US Dollar
        info.Name=iUnitName;
        info.Symbol=iUnitName;
        info.Country=i18nc ( "Noun, a country", "France" );
        info.Date=QDate ( 1987,1,1 );
        info.Value=-1;
        info.Internet="^FCHI";
    }
    else if ( iUnitName==i18nc ( "Noun, a currency", "NASDAQ" ) )
    {
        //NASDAQ
        info.Name=iUnitName;
        info.Symbol=iUnitName;
        info.Country=i18nc ( "Noun, a country", "United States" );
        info.Date=QDate ( 1971,2,5 );
        info.Value=-1;
        info.Internet="^IXIC";
    }
    else if ( iUnitName==i18nc ( "Noun, a currency", "Dow Jones (DJIA)" ) || iUnitName=="DJIA" )
    {
        //Dow Jones
        info.Name=iUnitName;
        info.Symbol="DJIA";
        info.Country=i18nc ( "Noun, a country", "United States" );
        info.Date=QDate ( 1884,1,1 );
        info.Value=-1;
        info.Internet="^DJI";
    }
    else if ( iUnitName==i18nc ( "Noun, a currency", "SBF 120" ) )
    {
        //SBF 120
        info.Name=iUnitName;
        info.Symbol=iUnitName;
        info.Country=i18nc ( "Noun, a country", "France" );
        info.Date=QDate ( 1990,12,31 );
        info.Value=-1;
        info.Internet="^SBF120";
    }

    return info;
}


SKGError SKGUnitObject::createCurrencyUnit ( const SKGDocumentBank* iDocument, const QString& iUnitName, SKGUnitObject& oUnit )
{
    SKGError err;
    if ( iDocument )
    {
        SKGUnitObject parentUnit;
        oUnit=SKGUnitObject ( iDocument );

        SKGUnitObject::UnitType type=SKGUnitObject::CURRENCY;
        SKGServices::SKGUnitInfo prim= ( ( SKGDocumentBank* ) iDocument )->getPrimaryUnit();
        SKGServices::SKGUnitInfo seco= ( ( SKGDocumentBank* ) iDocument )->getSecondaryUnit();

        //Get information on the unit
        SKGServices::SKGUnitInfo info=getUnitInfo ( iUnitName );
        if (info.Name.isEmpty()) err=SKGError(ERR_INVALIDARG, i18nc("Error message", "Unknown unit [%1]",iUnitName));
        if (err.isSucceeded() && !info.Parent.isEmpty()) err=createCurrencyUnit ( iDocument, info.Parent, parentUnit );

        //Set the type
        if ( info.Name==info.Symbol )
        {
            //This is an index
            type=SKGUnitObject::INDEX;
        }
        else if ( !info.Parent.isEmpty() )
        {
            //This is a secondary unit
            type= ( seco.Symbol.isEmpty() || seco.Symbol==info.Symbol  ? SKGUnitObject::SECONDARY : SKGUnitObject::CURRENCY );
        }
        else
        {
            //As primary
            type= ( prim.Symbol.isEmpty() || prim.Symbol==info.Symbol ? SKGUnitObject::PRIMARY : SKGUnitObject::CURRENCY );
        }

        //Point on primary unit
        if ( info.Value==1 && err.isSucceeded() && ( type==SKGUnitObject::CURRENCY || type==SKGUnitObject::SECONDARY ) )
        {
            SKGUnitObject primunit ( iDocument );
            err=primunit.setSymbol ( prim.Symbol );
            if ( err.isSucceeded() ) err=primunit.load();
            if ( err.isSucceeded() )
            {
                QString codeprimunit=getInternationalCode ( primunit.getName() );
                QString codeunit=getInternationalCode ( info.Name );
                if ( !codeprimunit.isEmpty() )
                {
                    info.Internet=codeunit+codeprimunit+"=X";
                    info.Value=-1;

                    parentUnit=SKGUnitObject ( iDocument );
                    err=parentUnit.setSymbol ( prim.Symbol );
                    if ( err.isSucceeded() ) err=parentUnit.load();
                }
            }

        }
        //TODO: other european currencies. see http://fr.wikipedia.org/wiki/Euro#Conversion_de_l.E2.80.99euro_dans_les_anciennes_devises_des_pays_membres

        if ( err.isSucceeded() ) err = oUnit.setName ( info.Name );
        if ( err.isSucceeded() && oUnit.exist() ) err=oUnit.load();
        if ( err.isSucceeded() ) err = oUnit.setType ( type );
        if ( err.isSucceeded() ) err = oUnit.setSymbol ( info.Symbol );
        if ( err.isSucceeded() ) err = oUnit.setInternetCode ( info.Internet );
        if ( err.isSucceeded() ) err = oUnit.setCountry ( info.Country );
        if ( err.isSucceeded() ) err = oUnit.setNumberDecimal ( info.NbDecimal );
        if ( err.isSucceeded() && parentUnit.exist() ) err = oUnit.setUnit ( parentUnit );
        if ( err.isSucceeded() ) err = oUnit.save();

        //Creation of the value
        if ( info.Value>0 )
        {
            SKGUnitValueObject unitValue;
            if ( err.isSucceeded() ) err = oUnit.addUnitValue ( unitValue );
            if ( err.isSucceeded() ) err = unitValue.setDate ( info.Date );
            if ( err.isSucceeded() ) err = unitValue.setQuantity ( info.Value );
            if ( err.isSucceeded() ) err = unitValue.save();
        }
    }
    return err;
}

double SKGUnitObject::convert ( double iValue, const SKGUnitObject& iUnitFrom, const SKGUnitObject& iUnitTo )
{
    double output=iValue;
    if ( iUnitFrom!=iUnitTo )
    {
        double valFrom=SKGServices::stringToDouble ( iUnitFrom.getAttribute ( "f_CURRENTAMOUNT" ) );
        double valTo=SKGServices::stringToDouble ( iUnitTo.getAttribute ( "f_CURRENTAMOUNT" ) );
        output=iValue*valFrom/valTo;
    }
    return output;
}

SKGError SKGUnitObject::setSymbol ( const QString& iSymbol )
{
    return setAttribute ( "t_symbol", iSymbol );
}

QString SKGUnitObject::getSymbol() const
{
    return getAttribute ( "t_symbol" );
}

SKGError SKGUnitObject::setNumberDecimal ( int iNb )
{
    return setAttribute ( "i_nbdecimal", SKGServices::intToString ( iNb ) );
}

int SKGUnitObject::getNumberDecimal() const
{
    return SKGServices::stringToInt ( getAttribute ( "i_nbdecimal" ) );
}

SKGError SKGUnitObject::setCountry ( const QString& iCountry )
{
    return setAttribute ( "t_country", iCountry );
}

QString SKGUnitObject::getCountry() const
{
    return getAttribute ( "t_country" );
}

SKGError SKGUnitObject::setInternetCode ( const QString& iCode )
{
    return setAttribute ( "t_internet_code", iCode );
}

QString SKGUnitObject::getInternetCode() const
{
    return getAttribute ( "t_internet_code" );
}

SKGError SKGUnitObject::setType ( SKGUnitObject::UnitType iType )
{
    SKGError err;
    if ( getAttribute ( "t_type" ).length() ==0 || this->getType() !=iType )
    {
        //Guaranty that PRIMARY and SECONDARY is unique
        if ( iType==PRIMARY || iType==SECONDARY )
        {
            //Set old SECONDARY as CURRENCY
            err=SKGServices::executeSqliteOrder ( getDocument(), "UPDATE unit SET t_type='C' WHERE t_type='2'" );

            //Set old PRIMARY as SECONDARY
            if ( err.isSucceeded() && iType==PRIMARY ) err=SKGServices::executeSqliteOrder ( getDocument(), "UPDATE unit SET t_type='2' WHERE t_type='1'" );
        }
    }
    if ( err.isSucceeded() ) err=setAttribute ( "t_type", ( iType==CURRENCY ? "C" : ( iType==PRIMARY ? "1" : ( iType==SECONDARY ? "2" : ( iType==SHARE ? "S" : ( iType==INDEX ? "I" : "O" ) ) ) ) ) );
    return err;
}

SKGUnitObject::UnitType SKGUnitObject::getType() const
{
    QString typeString=getAttribute ( "t_type" );
    return ( typeString=="C" ? CURRENCY : ( typeString=="S" ? SHARE : ( typeString=="1" ? PRIMARY : ( typeString=="2" ? SECONDARY : ( typeString=="I" ? INDEX : OBJECT ) ) ) ) );
}

SKGError SKGUnitObject::getUnit ( SKGUnitObject& oUnit ) const
{
    SKGError err = getObject ( getDocument(), "v_unit", "id=" + getAttribute ( "rd_unit_id" ), oUnit );
    return err;
}

SKGError SKGUnitObject::setUnit ( const SKGUnitObject& iUnit )
{
    SKGError err;
    if ( *this==iUnit ) err=SKGError ( ERR_INVALIDARG, i18nc ( "Error message", "Reference unit of a unit cannot be itself." ) );
    else
    {
        err=setAttribute ( "rd_unit_id", SKGServices::intToString ( iUnit.getID() ) );
    }
    return err;
}

SKGError SKGUnitObject::removeUnit()
{
    return setAttribute ( "rd_unit_id", "0" );
}

SKGError SKGUnitObject::addUnitValue ( SKGUnitValueObject& oUnitValue )
{
    SKGError err;
    if ( getID() == 0 ) err=SKGError ( ERR_FAIL, i18nc ( "Error message", "%1 failed because linked object is not yet saved in the database.",QString ( "SKGUnitObject::addUnitValue" ) ) );
    else
    {
        oUnitValue = SKGUnitValueObject ( ( SKGDocumentBank* ) getDocument() );
        err = oUnitValue.setAttribute ( "rd_unit_id", SKGServices::intToString ( getID() ) );
    }
    return err;
}

SKGError SKGUnitObject::getUnitValues ( SKGListSKGObjectBase& oUnitValueList ) const
{
    SKGError err = getObjects ( getDocument(), "v_unitvalue",
                                "rd_unit_id=" + SKGServices::intToString ( getID() ),
                                oUnitValueList );
    return err;
}

SKGError SKGUnitObject::getLastUnitValue ( SKGUnitValueObject& oUnitValue ) const
{
    return SKGObjectBase::getObject ( getDocument(), "v_unitvalue",
                                      "rd_unit_id=" + SKGServices::intToString ( getID() ) +" AND d_date=(select MAX(u2.d_date) from unitvalue u2 where u2.rd_unit_id=" + SKGServices::intToString ( getID() ) + ')',
                                      oUnitValue );
}

SKGError SKGUnitObject::getUnitValue ( const QDate& iDate, SKGUnitValueObject& oUnitValue ) const
{
    QString ids=SKGServices::intToString ( getID() );
    QString dates=SKGServices::dateToSqlString ( QDateTime ( iDate ) );
    SKGError err=SKGObjectBase::getObject ( getDocument(), "v_unitvalue",
                                            "rd_unit_id=" + ids +" AND d_date<='"+dates+
                                            "' AND  ABS(strftime('%s','"+dates+
                                            "')-strftime('%s',d_date))=(select MIN(ABS(strftime('%s','"+dates+
                                            "')-strftime('%s',u2.d_date))) from unitvalue u2 where u2.rd_unit_id=" + ids +
                                            " AND u2.d_date<='"+dates+"')",
                                            oUnitValue );

    //If not found then get first
    if ( err.isFailed() ) err=SKGObjectBase::getObject ( getDocument(), "v_unitvalue",
                                  "rd_unit_id=" + SKGServices::intToString ( getID() ) +" AND d_date=(select MIN(d_date) from unitvalue where rd_unit_id=" +
                                  SKGServices::intToString ( getID() ) + ')',
                                  oUnitValue );
    return err;
}

double SKGUnitObject::getAmount ( const QDate& iDate ) const
{
    double output=0;

    //Search result in cache
    QString ids=SKGServices::intToString ( getID() );
    QString dates=SKGServices::dateToSqlString ( QDateTime ( iDate ) );
    QString key="unitvalue-"+ids+'-'+dates;
    QString val=getDocument()->getCachedValue ( key );
    if ( val.isEmpty() )
    {
        //Get quantity
        double quantity=1;
        SKGUnitValueObject uv;
        if ( getUnitValue ( iDate, uv ).isSucceeded() ) quantity=uv.getQuantity();

        SKGUnitObject unit;
        double coef=1;
        if ( getUnit ( unit ).isSucceeded() ) coef=unit.getAmount ( iDate );

        output=coef*quantity;
        ( ( SKGDocument* ) getDocument() )->addValueInCache ( key, SKGServices::doubleToString ( output ) );

        if ( getAttribute ( "i_NBVALUES" ) =="1" )
        {
            //Store value for this symbol for all date
            ( ( SKGDocument* ) getDocument() )->addValueInCache ( "unitvalue-"+ids, SKGServices::doubleToString ( output ) );
        }
    }
    else
    {
        output=SKGServices::stringToDouble ( val );
    }
    return output;

}

double SKGUnitObject::getDailyChange ( const QDate& iDate ) const
{
    double output=0;
    SKGStringListList result;
    SKGError err=SKGServices::executeSelectSqliteOrder ( getDocument(),
                 "SELECT d_date, f_quantity from unitvalue where rd_unit_id="+
                 SKGServices::intToString ( getID() ) +
                 " AND d_date<='"+SKGServices::dateToSqlString ( QDateTime ( iDate ) ) +
                 "' ORDER BY d_date DESC LIMIT 2",
                 result );
    if ( err.isSucceeded() && result.count() ==3 )
    {
        double v2=SKGServices::stringToDouble ( result.at ( 1 ).at ( 1 ) );
        double v1=SKGServices::stringToDouble ( result.at ( 2 ).at ( 1 ) );

        QDate d2=SKGServices::stringToTime ( result.at ( 1 ).at ( 0 ) ).date();
        QDate d1=SKGServices::stringToTime ( result.at ( 2 ).at ( 0 ) ).date();

        output=100* ( exp ( log ( v2/v1 ) / ( d1.daysTo ( d2 ) ) )-1 );
    }
    return output;
}

SKGError SKGUnitObject::split ( double iRatio ) const
{
    SKGError err;
    if ( iRatio>0 )
    {

        err=SKGServices::executeSqliteOrder ( getDocument(), "UPDATE unitvalue SET f_quantity=f_quantity/"+SKGServices::doubleToString ( iRatio ) +
                                              " WHERE rd_unit_id="+SKGServices::intToString ( getID() ) );
        if ( err.isSucceeded() ) err=SKGServices::executeSqliteOrder ( getDocument(), "UPDATE suboperation SET f_value=f_value*"+SKGServices::doubleToString ( iRatio ) +
                                         " WHERE rd_operation_id IN (SELECT id FROM operation WHERE rc_unit_id="+SKGServices::intToString ( getID() ) +')' );
    }
    else err=SKGError ( ERR_INVALIDARG, i18nc ( "Error message", "Invalid ratio. Ratio must be greater than 0." ) );
    return err;
}

#include "skgunitobject.moc"
